# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    ApplicationTestCaseOutputListResponse,
    ApplicationTestCaseOutputBatchResponse,
    ApplicationTestCaseOutputRetrieveResponse,
)
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestApplicationTestCaseOutputs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        application_test_case_output = client.application_test_case_outputs.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        )
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        application_test_case_output = client.application_test_case_outputs.retrieve(
            application_test_case_output_id="application_test_case_output_id",
            view=["MetricScores", "TestCaseVersion", "Trace"],
        )
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.application_test_case_outputs.with_raw_response.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = response.parse()
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.application_test_case_outputs.with_streaming_response.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = response.parse()
            assert_matches_type(
                ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_test_case_output_id` but received ''"
        ):
            client.application_test_case_outputs.with_raw_response.retrieve(
                application_test_case_output_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        application_test_case_output = client.application_test_case_outputs.list()
        assert_matches_type(
            SyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        application_test_case_output = client.application_test_case_outputs.list(
            account_id="account_id",
            application_variant_id=0,
            application_variant_report_id=0,
            evaluation_dataset_id=0,
            evaluation_dataset_version_num=0,
            limit=1,
            page=1,
            view=["MetricScores", "TestCaseVersion", "Trace"],
        )
        assert_matches_type(
            SyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.application_test_case_outputs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = response.parse()
        assert_matches_type(
            SyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.application_test_case_outputs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = response.parse()
            assert_matches_type(
                SyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_batch(self, client: SGPClient) -> None:
        application_test_case_output = client.application_test_case_outputs.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        )
        assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

    @parametrize
    def test_raw_response_batch(self, client: SGPClient) -> None:
        response = client.application_test_case_outputs.with_raw_response.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = response.parse()
        assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

    @parametrize
    def test_streaming_response_batch(self, client: SGPClient) -> None:
        with client.application_test_case_outputs.with_streaming_response.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = response.parse()
            assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncApplicationTestCaseOutputs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        application_test_case_output = await async_client.application_test_case_outputs.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        )
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application_test_case_output = await async_client.application_test_case_outputs.retrieve(
            application_test_case_output_id="application_test_case_output_id",
            view=["MetricScores", "TestCaseVersion", "Trace"],
        )
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_test_case_outputs.with_raw_response.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = await response.parse()
        assert_matches_type(ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_test_case_outputs.with_streaming_response.retrieve(
            application_test_case_output_id="application_test_case_output_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = await response.parse()
            assert_matches_type(
                ApplicationTestCaseOutputRetrieveResponse, application_test_case_output, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_test_case_output_id` but received ''"
        ):
            await async_client.application_test_case_outputs.with_raw_response.retrieve(
                application_test_case_output_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        application_test_case_output = await async_client.application_test_case_outputs.list()
        assert_matches_type(
            AsyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application_test_case_output = await async_client.application_test_case_outputs.list(
            account_id="account_id",
            application_variant_id=0,
            application_variant_report_id=0,
            evaluation_dataset_id=0,
            evaluation_dataset_version_num=0,
            limit=1,
            page=1,
            view=["MetricScores", "TestCaseVersion", "Trace"],
        )
        assert_matches_type(
            AsyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_test_case_outputs.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = await response.parse()
        assert_matches_type(
            AsyncPageResponse[ApplicationTestCaseOutputListResponse], application_test_case_output, path=["response"]
        )

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_test_case_outputs.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = await response.parse()
            assert_matches_type(
                AsyncPageResponse[ApplicationTestCaseOutputListResponse],
                application_test_case_output,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_batch(self, async_client: AsyncSGPClient) -> None:
        application_test_case_output = await async_client.application_test_case_outputs.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        )
        assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

    @parametrize
    async def test_raw_response_batch(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_test_case_outputs.with_raw_response.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_test_case_output = await response.parse()
        assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

    @parametrize
    async def test_streaming_response_batch(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_test_case_outputs.with_streaming_response.batch(
            items=[
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
                {
                    "account_id": "account_id",
                    "application_variant_id": "application_variant_id",
                    "evaluation_dataset_version_num": 0,
                    "output": {"generation_output": "generation_output"},
                    "test_case_id": "test_case_id",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_test_case_output = await response.parse()
            assert_matches_type(ApplicationTestCaseOutputBatchResponse, application_test_case_output, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types.applications import ChatThreadHistory
from scale_gp.types.chat_threads import MessageUpdateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMessages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        message = client.chat_threads.messages.retrieve(
            "thread_id",
        )
        assert_matches_type(ChatThreadHistory, message, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.chat_threads.messages.with_raw_response.retrieve(
            "thread_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(ChatThreadHistory, message, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.chat_threads.messages.with_streaming_response.retrieve(
            "thread_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(ChatThreadHistory, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.chat_threads.messages.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        message = client.chat_threads.messages.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        )
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        message = client.chat_threads.messages.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
            output_text="output_text",
        )
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.chat_threads.messages.with_raw_response.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = response.parse()
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.chat_threads.messages.with_streaming_response.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = response.parse()
            assert_matches_type(MessageUpdateResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.chat_threads.messages.with_raw_response.update(
                application_interaction_id="application_interaction_id",
                thread_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_interaction_id` but received ''"
        ):
            client.chat_threads.messages.with_raw_response.update(
                application_interaction_id="",
                thread_id="thread_id",
            )


class TestAsyncMessages:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        message = await async_client.chat_threads.messages.retrieve(
            "thread_id",
        )
        assert_matches_type(ChatThreadHistory, message, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chat_threads.messages.with_raw_response.retrieve(
            "thread_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(ChatThreadHistory, message, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chat_threads.messages.with_streaming_response.retrieve(
            "thread_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(ChatThreadHistory, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.chat_threads.messages.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        message = await async_client.chat_threads.messages.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        )
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        message = await async_client.chat_threads.messages.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
            output_text="output_text",
        )
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chat_threads.messages.with_raw_response.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        message = await response.parse()
        assert_matches_type(MessageUpdateResponse, message, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chat_threads.messages.with_streaming_response.update(
            application_interaction_id="application_interaction_id",
            thread_id="thread_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            message = await response.parse()
            assert_matches_type(MessageUpdateResponse, message, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.chat_threads.messages.with_raw_response.update(
                application_interaction_id="application_interaction_id",
                thread_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_interaction_id` but received ''"
        ):
            await async_client.chat_threads.messages.with_raw_response.update(
                application_interaction_id="",
                thread_id="thread_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["SynthesizeChunksResponse", "SourceNode", "SourceNodeNode", "SourceNodeNodeRelationships"]


class SourceNodeNodeRelationships(BaseModel):
    hash: str

    metadata: object

    node_id: str

    node_type: Optional[str] = None


class SourceNodeNode(BaseModel):
    text: str

    embedding: Optional[List[float]] = None

    end_char_idx: Optional[int] = None

    extra_info: Optional[object] = None

    hash: Optional[str] = None

    id: Optional[str] = FieldInfo(alias="id_", default=None)

    relationships: Optional[Dict[str, SourceNodeNodeRelationships]] = None

    start_char_idx: Optional[int] = None


class SourceNode(BaseModel):
    node: SourceNodeNode

    score: float


class SynthesizeChunksResponse(BaseModel):
    response: str
    """Natural language response addressing the query."""

    metadata: Optional[Dict[str, object]] = None
    """Optional metadata present on each chunk."""

    source_nodes: Optional[List[SourceNode]] = None
    """List of chunks used to synthesize the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["InstallDeploymentPackageResponse", "PackageInstallation"]


class PackageInstallation(BaseModel):
    id: str
    """The unique identifier of the entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    organization_id: str
    """The identifier of the organization."""

    package_version_id: str
    """The package version which is installed"""

    account_id: Optional[str] = None
    """The ID of the account to which the installed package belongs.

    Unset for non-EGP entity deployment packages.
    """

    deleted_at: Optional[datetime] = None
    """The date and time when the entity was deleted in ISO format."""

    install_log: Optional[str] = None
    """output of installation step"""

    state: Optional[object] = None
    """Properties of the installation, must match schema of the package type."""

    status: Optional[Literal["INSTALLED", "UNINSTALLED", "IN_PROGRESS", "FAILED"]] = None
    """An enumeration."""


class InstallDeploymentPackageResponse(BaseModel):
    package_installation: PackageInstallation



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from . import evaluation_dataset as _evaluation_dataset
from .._models import BaseModel

__all__ = [
    "ApplicationVariantWithScores",
    "CategoryScore",
    "CategoryScoreApplicationCategoryScoreAccuracy",
    "CategoryScoreApplicationCategoryScoreAccuracyMetricScore",
    "CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerCorrectness",
    "CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerRelevance",
    "CategoryScoreApplicationCategoryScoreRetrieval",
    "CategoryScoreApplicationCategoryScoreRetrievalMetricScore",
    "CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreFaithfulness",
    "CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreContextRecall",
    "CategoryScoreApplicationCategoryScoreQuality",
    "CategoryScoreApplicationCategoryScoreQualityMetricScore",
    "CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreCoherence",
    "CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreGrammar",
    "CategoryScoreApplicationCategoryScoreTrustAndSafety",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScore",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafety",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScore",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumBiasAndStereotypingSafetyBiasAndStereotyping",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumOpinionsDisputedTopicsSafetyOpinionsDisputedTopics",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumUnethicalHarmfulActivitiesSafetyUnethicalHarmfulActivities",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumCopyrightViolationsSafetyCopyrightViolations",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumHarmfulContentSafetyHarmfulContent",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumPrivacyViolationsSafetyPrivacyViolations",
    "CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreModeration",
    "EvaluationDataset",
]


class CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerCorrectness(BaseModel):
    category: Literal["accuracy"]

    metric_type: Literal["answer-correctness"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerRelevance(BaseModel):
    category: Literal["accuracy"]

    metric_type: Literal["answer-relevance"]

    score: Optional[float] = None


CategoryScoreApplicationCategoryScoreAccuracyMetricScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerCorrectness,
    CategoryScoreApplicationCategoryScoreAccuracyMetricScoreApplicationMetricScoreAnswerRelevance,
]


class CategoryScoreApplicationCategoryScoreAccuracy(BaseModel):
    category: Literal["accuracy"]

    metric_scores: List[CategoryScoreApplicationCategoryScoreAccuracyMetricScore]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreFaithfulness(BaseModel):
    category: Literal["retrieval"]

    metric_type: Literal["faithfulness"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreContextRecall(BaseModel):
    category: Literal["retrieval"]

    metric_type: Literal["context-recall"]

    score: Optional[float] = None


CategoryScoreApplicationCategoryScoreRetrievalMetricScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreFaithfulness,
    CategoryScoreApplicationCategoryScoreRetrievalMetricScoreApplicationMetricScoreContextRecall,
]


class CategoryScoreApplicationCategoryScoreRetrieval(BaseModel):
    category: Literal["retrieval"]

    metric_scores: List[CategoryScoreApplicationCategoryScoreRetrievalMetricScore]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreCoherence(BaseModel):
    category: Literal["quality"]

    metric_type: Literal["coherence"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreGrammar(BaseModel):
    category: Literal["quality"]

    metric_type: Literal["grammar"]

    score: Optional[float] = None


CategoryScoreApplicationCategoryScoreQualityMetricScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreCoherence,
    CategoryScoreApplicationCategoryScoreQualityMetricScoreApplicationMetricScoreGrammar,
]


class CategoryScoreApplicationCategoryScoreQuality(BaseModel):
    category: Literal["quality"]

    metric_scores: List[CategoryScoreApplicationCategoryScoreQualityMetricScore]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumBiasAndStereotypingSafetyBiasAndStereotyping(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-bias-and-stereotyping"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumOpinionsDisputedTopicsSafetyOpinionsDisputedTopics(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-opinions-disputed-topics"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumUnethicalHarmfulActivitiesSafetyUnethicalHarmfulActivities(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-unethical-harmful-activities"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumCopyrightViolationsSafetyCopyrightViolations(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-copyright-violations"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumHarmfulContentSafetyHarmfulContent(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-harmful-content"]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumPrivacyViolationsSafetyPrivacyViolations(
    BaseModel
):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety-privacy-violations"]

    score: Optional[float] = None


CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumBiasAndStereotypingSafetyBiasAndStereotyping,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumOpinionsDisputedTopicsSafetyOpinionsDisputedTopics,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumUnethicalHarmfulActivitiesSafetyUnethicalHarmfulActivities,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumCopyrightViolationsSafetyCopyrightViolations,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumHarmfulContentSafetyHarmfulContent,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScoreApplicationMetricScoreLiteralApplicationScoreCategoryEnumTrustAndSafetyTrustAndSafetyLiteralSafetySubMetricTypeEnumPrivacyViolationsSafetyPrivacyViolations,
]


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafety(BaseModel):
    category: Literal["trust-and-safety"]

    metric_type: Literal["safety"]

    sub_metric_scores: List[
        CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafetySubMetricScore
    ]

    score: Optional[float] = None


class CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreModeration(BaseModel):
    category: Literal["trust-and-safety"]

    metric_type: Literal["moderation"]

    score: Optional[float] = None


CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreSafety,
    CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScoreApplicationMetricScoreModeration,
]


class CategoryScoreApplicationCategoryScoreTrustAndSafety(BaseModel):
    category: Literal["trust-and-safety"]

    metric_scores: List[CategoryScoreApplicationCategoryScoreTrustAndSafetyMetricScore]

    score: Optional[float] = None


CategoryScore: TypeAlias = Union[
    CategoryScoreApplicationCategoryScoreAccuracy,
    CategoryScoreApplicationCategoryScoreRetrieval,
    CategoryScoreApplicationCategoryScoreQuality,
    CategoryScoreApplicationCategoryScoreTrustAndSafety,
]


class EvaluationDataset(BaseModel):
    evaluation_dataset: _evaluation_dataset.EvaluationDataset

    evaluation_dataset_version_num: int

    generation_status: Literal["Pending", "Running", "Completed", "Failed", "Canceled"]
    """An enumeration."""

    scored_test_case_count: Optional[int] = None


class ApplicationVariantWithScores(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    application_spec_id: str

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    category_scores: Optional[List[CategoryScore]] = None

    evaluation_datasets: Optional[List[EvaluationDataset]] = None

    score: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from ..._models import BaseModel

__all__ = ["ModelUsage", "Data"]


class Data(BaseModel):
    date: datetime
    """The start date of the data point"""

    prompt_tokens: int
    """The number of prompt tokens used"""

    response_tokens: int
    """The number of response tokens used"""

    summary_tokens: int
    """The number of all the tokens used"""


class ModelUsage(BaseModel):
    data: List[Data]
    """The list of data points for the given period"""

    end_date: datetime
    """The end date of the data points. Equal to the last date in the data list"""

    start_date: datetime
    """The start date of the data points. Equal to the first date in the data list"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias, TypedDict

from .s3_data_source_auth_config_param import S3DataSourceAuthConfigParam
from .slack_data_source_auth_config_param import SlackDataSourceAuthConfigParam
from .confluence_data_source_auth_config_param import ConfluenceDataSourceAuthConfigParam
from .share_point_data_source_auth_config_param import SharePointDataSourceAuthConfigParam
from .google_drive_data_source_auth_config_param import GoogleDriveDataSourceAuthConfigParam
from .azure_blob_storage_data_source_auth_config_param import AzureBlobStorageDataSourceAuthConfigParam

__all__ = ["KnowledgeBaseDataSourceUpdateParams", "DataSourceAuthConfig"]


class KnowledgeBaseDataSourceUpdateParams(TypedDict, total=False):
    data_source_auth_config: DataSourceAuthConfig

    description: str

    name: str


DataSourceAuthConfig: TypeAlias = Union[
    SharePointDataSourceAuthConfigParam,
    AzureBlobStorageDataSourceAuthConfigParam,
    GoogleDriveDataSourceAuthConfigParam,
    S3DataSourceAuthConfigParam,
    ConfluenceDataSourceAuthConfigParam,
    SlackDataSourceAuthConfigParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["CreateAccountResponse"]


class CreateAccountResponse(BaseModel):
    account_id: str
    """The ID of the created account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import evaluation_config_list_params, evaluation_config_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.evaluation_config import EvaluationConfig
from ..types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["EvaluationConfigsResource", "AsyncEvaluationConfigsResource"]


class EvaluationConfigsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluationConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EvaluationConfigsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        evaluation_type: Literal["studio", "llm_auto", "human", "llm_benchmark"],
        question_set_id: str,
        studio_project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationConfig:
        """
        ### Description

        Creates a evaluation config

        ### Details

        This API can be used to create a evaluation config. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        config.

        Args:
          account_id: The ID of the account that owns the given entity.

          evaluation_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/evaluation-configs",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "evaluation_type": evaluation_type,
                    "question_set_id": question_set_id,
                    "studio_project_id": studio_project_id,
                },
                evaluation_config_create_params.EvaluationConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationConfig,
        )

    def retrieve(
        self,
        evaluation_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationConfig:
        """
        ### Description

        Gets the details of a evaluation config

        ### Details

        This API can be used to get information about a single evaluation config by ID.
        To use this API, pass in the `id` that was returned from your Create Evaluation
        Config API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_config_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_config_id` but received {evaluation_config_id!r}"
            )
        return self._get(
            f"/v4/evaluation-configs/{evaluation_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationConfig,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[EvaluationConfig]:
        """
        ### Description

        Lists all evaluation configs accessible to the user.

        ### Details

        This API can be used to list evaluation configs. If a user has access to
        multiple accounts, all evaluation configs from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluation-configs",
            page=SyncPageResponse[EvaluationConfig],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_config_list_params.EvaluationConfigListParams,
                ),
            ),
            model=EvaluationConfig,
        )

    def delete(
        self,
        evaluation_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a evaluation config

        ### Details

        This API can be used to delete a evaluation config by ID. To use this API, pass
        in the `id` that was returned from your Create Evaluation Config API call as a
        path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_config_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_config_id` but received {evaluation_config_id!r}"
            )
        return self._delete(
            f"/v4/evaluation-configs/{evaluation_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncEvaluationConfigsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluationConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEvaluationConfigsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        evaluation_type: Literal["studio", "llm_auto", "human", "llm_benchmark"],
        question_set_id: str,
        studio_project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationConfig:
        """
        ### Description

        Creates a evaluation config

        ### Details

        This API can be used to create a evaluation config. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        config.

        Args:
          account_id: The ID of the account that owns the given entity.

          evaluation_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/evaluation-configs",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "evaluation_type": evaluation_type,
                    "question_set_id": question_set_id,
                    "studio_project_id": studio_project_id,
                },
                evaluation_config_create_params.EvaluationConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationConfig,
        )

    async def retrieve(
        self,
        evaluation_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationConfig:
        """
        ### Description

        Gets the details of a evaluation config

        ### Details

        This API can be used to get information about a single evaluation config by ID.
        To use this API, pass in the `id` that was returned from your Create Evaluation
        Config API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_config_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_config_id` but received {evaluation_config_id!r}"
            )
        return await self._get(
            f"/v4/evaluation-configs/{evaluation_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationConfig,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EvaluationConfig, AsyncPageResponse[EvaluationConfig]]:
        """
        ### Description

        Lists all evaluation configs accessible to the user.

        ### Details

        This API can be used to list evaluation configs. If a user has access to
        multiple accounts, all evaluation configs from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluation-configs",
            page=AsyncPageResponse[EvaluationConfig],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_config_list_params.EvaluationConfigListParams,
                ),
            ),
            model=EvaluationConfig,
        )

    async def delete(
        self,
        evaluation_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a evaluation config

        ### Details

        This API can be used to delete a evaluation config by ID. To use this API, pass
        in the `id` that was returned from your Create Evaluation Config API call as a
        path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_config_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_config_id` but received {evaluation_config_id!r}"
            )
        return await self._delete(
            f"/v4/evaluation-configs/{evaluation_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class EvaluationConfigsResourceWithRawResponse:
    def __init__(self, evaluation_configs: EvaluationConfigsResource) -> None:
        self._evaluation_configs = evaluation_configs

        self.create = to_raw_response_wrapper(
            evaluation_configs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evaluation_configs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            evaluation_configs.list,
        )
        self.delete = to_raw_response_wrapper(
            evaluation_configs.delete,
        )


class AsyncEvaluationConfigsResourceWithRawResponse:
    def __init__(self, evaluation_configs: AsyncEvaluationConfigsResource) -> None:
        self._evaluation_configs = evaluation_configs

        self.create = async_to_raw_response_wrapper(
            evaluation_configs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evaluation_configs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            evaluation_configs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            evaluation_configs.delete,
        )


class EvaluationConfigsResourceWithStreamingResponse:
    def __init__(self, evaluation_configs: EvaluationConfigsResource) -> None:
        self._evaluation_configs = evaluation_configs

        self.create = to_streamed_response_wrapper(
            evaluation_configs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evaluation_configs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            evaluation_configs.list,
        )
        self.delete = to_streamed_response_wrapper(
            evaluation_configs.delete,
        )


class AsyncEvaluationConfigsResourceWithStreamingResponse:
    def __init__(self, evaluation_configs: AsyncEvaluationConfigsResource) -> None:
        self._evaluation_configs = evaluation_configs

        self.create = async_to_streamed_response_wrapper(
            evaluation_configs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evaluation_configs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluation_configs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            evaluation_configs.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["EvaluationConfig"]


class EvaluationConfig(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    evaluation_type: Literal["studio", "llm_auto", "human", "llm_benchmark"]
    """An enumeration."""

    question_set_id: str

    studio_project_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._compat import PYDANTIC_V2, ConfigDict
from .._models import BaseModel
from .parameter_schema import ParameterSchema

__all__ = [
    "ModelTemplate",
    "VendorConfiguration",
    "VendorConfigurationBundleConfig",
    "VendorConfigurationEndpointConfig",
    "VendorConfigurationFineTuningJobBundleConfig",
    "VendorConfigurationFineTuningJobBundleConfigResources",
]


class VendorConfigurationBundleConfig(BaseModel):
    image: str

    registry: str

    tag: str

    command: Optional[List[str]] = None

    env: Optional[Dict[str, str]] = None

    healthcheck_route: Optional[str] = None

    predict_route: Optional[str] = None

    readiness_initial_delay_seconds: Optional[int] = None

    streaming_command: Optional[List[str]] = None

    streaming_predict_route: Optional[str] = None


class VendorConfigurationEndpointConfig(BaseModel):
    cpus: Optional[int] = None

    endpoint_type: Optional[Literal["SYNC", "ASYNC", "STREAMING", "BATCH"]] = None
    """An enum representing the different types of model endpoint types supported.

    Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
    that the model endpoint type is async. STREAMING: Denotes that the model
    endpoint type is streaming. BATCH: Denotes that the model endpoint type is
    batch.
    """

    gpu_type: Optional[
        Literal[
            "nvidia-tesla-t4",
            "nvidia-ampere-a10",
            "nvidia-ampere-a100",
            "nvidia-ampere-a100e",
            "nvidia-hopper-h100",
            "nvidia-hopper-h100-1g20gb",
            "nvidia-hopper-h100-3g40gb",
        ]
    ] = None
    """An enumeration."""

    gpus: Optional[int] = None

    high_priority: Optional[bool] = None

    max_workers: Optional[int] = None

    memory: Optional[str] = None

    min_workers: Optional[int] = None

    per_worker: Optional[int] = None
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    storage: Optional[str] = None


class VendorConfigurationFineTuningJobBundleConfigResources(BaseModel):
    cpus: Optional[int] = None

    gpu_type: Optional[
        Literal[
            "nvidia-tesla-t4",
            "nvidia-ampere-a10",
            "nvidia-ampere-a100",
            "nvidia-ampere-a100e",
            "nvidia-hopper-h100",
            "nvidia-hopper-h100-1g20gb",
            "nvidia-hopper-h100-3g40gb",
        ]
    ] = None
    """An enumeration."""

    gpus: Optional[int] = None

    memory: Optional[str] = None

    storage: Optional[str] = None


class VendorConfigurationFineTuningJobBundleConfig(BaseModel):
    image: str

    registry: str

    tag: str

    command: Optional[List[str]] = None

    env: Optional[Dict[str, str]] = None

    mount_location: Optional[str] = None
    """
    The filesystem location where the fine tuning job's configuration will be
    available when it is started.
    """

    resources: Optional[VendorConfigurationFineTuningJobBundleConfigResources] = None

    training_dataset_schema_type: Optional[Literal["GENERATION", "RERANKING_QUESTIONS"]] = None
    """An enumeration."""


class VendorConfiguration(BaseModel):
    bundle_config: VendorConfigurationBundleConfig

    endpoint_config: Optional[VendorConfigurationEndpointConfig] = None

    fine_tuning_job_bundle_config: Optional[VendorConfigurationFineTuningJobBundleConfig] = None

    vendor: Optional[Literal["LAUNCH"]] = None


class ModelTemplate(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    endpoint_type: Literal["SYNC", "ASYNC", "STREAMING", "BATCH"]
    """An enum representing the different types of model endpoint types supported.

    Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
    that the model endpoint type is async. STREAMING: Denotes that the model
    endpoint type is streaming. BATCH: Denotes that the model endpoint type is
    batch.
    """

    model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]
    """An enum representing the different types of models supported.

    Attributes: COMPLETION: Denotes that the model type is completion.
    CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
    that the model type is agent. EMBEDDING: Denotes that the model type is
    embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
    that the model type is generic.
    """

    name: str

    vendor_configuration: VendorConfiguration
    """
    Configuration for launching a model using the Launch service which is an
    internal and self-hosted service developed by Scale that deploys models on
    Kubernetes.

    Attributes: vendor: The vendor of the model template bundle_config: The bundle
    configuration of the model template endpoint_config: The endpoint configuration
    of the model template
    """

    endpoint_protocol: Optional[Literal["SGP", "COHERE", "VLLM"]] = None
    """An enumeration."""

    model_creation_parameters_schema: Optional[ParameterSchema] = None
    """
    The field names and types of available parameter fields which may be specified
    during model creation
    """

    model_request_parameters_schema: Optional[ParameterSchema] = None
    """
    The field names and types of available parameter fields which may be specified
    in a model execution API's `model_request_parameters` field.
    """

    if PYDANTIC_V2:
        # allow fields with a `model_` prefix
        model_config = ConfigDict(protected_namespaces=tuple())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .test_cases import (
    TestCasesResource,
    AsyncTestCasesResource,
    TestCasesResourceWithRawResponse,
    AsyncTestCasesResourceWithRawResponse,
    TestCasesResourceWithStreamingResponse,
    AsyncTestCasesResourceWithStreamingResponse,
)
from .generation_jobs import (
    GenerationJobsResource,
    AsyncGenerationJobsResource,
    GenerationJobsResourceWithRawResponse,
    AsyncGenerationJobsResourceWithRawResponse,
    GenerationJobsResourceWithStreamingResponse,
    AsyncGenerationJobsResourceWithStreamingResponse,
)
from .evaluation_datasets import (
    EvaluationDatasetsResource,
    AsyncEvaluationDatasetsResource,
    EvaluationDatasetsResourceWithRawResponse,
    AsyncEvaluationDatasetsResourceWithRawResponse,
    EvaluationDatasetsResourceWithStreamingResponse,
    AsyncEvaluationDatasetsResourceWithStreamingResponse,
)
from .evaluation_dataset_versions import (
    EvaluationDatasetVersionsResource,
    AsyncEvaluationDatasetVersionsResource,
    EvaluationDatasetVersionsResourceWithRawResponse,
    AsyncEvaluationDatasetVersionsResourceWithRawResponse,
    EvaluationDatasetVersionsResourceWithStreamingResponse,
    AsyncEvaluationDatasetVersionsResourceWithStreamingResponse,
)
from .autogenerated_draft_test_cases import (
    AutogeneratedDraftTestCasesResource,
    AsyncAutogeneratedDraftTestCasesResource,
    AutogeneratedDraftTestCasesResourceWithRawResponse,
    AsyncAutogeneratedDraftTestCasesResourceWithRawResponse,
    AutogeneratedDraftTestCasesResourceWithStreamingResponse,
    AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse,
)

__all__ = [
    "EvaluationDatasetVersionsResource",
    "AsyncEvaluationDatasetVersionsResource",
    "EvaluationDatasetVersionsResourceWithRawResponse",
    "AsyncEvaluationDatasetVersionsResourceWithRawResponse",
    "EvaluationDatasetVersionsResourceWithStreamingResponse",
    "AsyncEvaluationDatasetVersionsResourceWithStreamingResponse",
    "TestCasesResource",
    "AsyncTestCasesResource",
    "TestCasesResourceWithRawResponse",
    "AsyncTestCasesResourceWithRawResponse",
    "TestCasesResourceWithStreamingResponse",
    "AsyncTestCasesResourceWithStreamingResponse",
    "AutogeneratedDraftTestCasesResource",
    "AsyncAutogeneratedDraftTestCasesResource",
    "AutogeneratedDraftTestCasesResourceWithRawResponse",
    "AsyncAutogeneratedDraftTestCasesResourceWithRawResponse",
    "AutogeneratedDraftTestCasesResourceWithStreamingResponse",
    "AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse",
    "GenerationJobsResource",
    "AsyncGenerationJobsResource",
    "GenerationJobsResourceWithRawResponse",
    "AsyncGenerationJobsResourceWithRawResponse",
    "GenerationJobsResourceWithStreamingResponse",
    "AsyncGenerationJobsResourceWithStreamingResponse",
    "EvaluationDatasetsResource",
    "AsyncEvaluationDatasetsResource",
    "EvaluationDatasetsResourceWithRawResponse",
    "AsyncEvaluationDatasetsResourceWithRawResponse",
    "EvaluationDatasetsResourceWithStreamingResponse",
    "AsyncEvaluationDatasetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse
from scale_gp.types.evaluation_datasets import (
    AutoGeneratedDraftTestCase,
    AutogeneratedDraftTestCaseListResponse,
    ApproveAutoGeneratedDraftTestCaseResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAutogeneratedDraftTestCases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            schema_type="GENERATION",
            topic_str="topic_str",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.create(
                evaluation_dataset_id="",
                account_id="account_id",
                approved=True,
                test_case_data={
                    "artifact_ids_filter": ["string", "string", "string"],
                    "input": "input",
                },
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            approved=True,
            schema_type="GENERATION",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(
            SyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
            check_missing_chunks=True,
            limit=1,
            page=1,
        )
        assert_matches_type(
            SyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(
            SyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(
                SyncPageResponse[AutogeneratedDraftTestCaseListResponse],
                autogenerated_draft_test_case,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.list(
                evaluation_dataset_id="",
            )

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_approve(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_method_approve_with_all_params(self, client: SGPClient) -> None:
        autogenerated_draft_test_case = client.evaluation_datasets.autogenerated_draft_test_cases.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            force=True,
        )
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_raw_response_approve(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = response.parse()
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    def test_streaming_response_approve(self, client: SGPClient) -> None:
        with client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = response.parse()
            assert_matches_type(
                ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_approve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )


class TestAsyncAutogeneratedDraftTestCases:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
            schema_type="GENERATION",
            topic_str="topic_str",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            approved=True,
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.create(
                evaluation_dataset_id="",
                account_id="account_id",
                approved=True,
                test_case_data={
                    "artifact_ids_filter": ["string", "string", "string"],
                    "input": "input",
                },
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.retrieve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.retrieve(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            approved=True,
            schema_type="GENERATION",
            test_case_data={
                "artifact_ids_filter": ["string", "string", "string"],
                "input": "input",
                "expected_extra_info": {
                    "chunks": [
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                        {
                            "metadata": {},
                            "text": "text",
                        },
                    ],
                    "schema_type": "CHUNKS",
                },
                "expected_output": "expected_output",
            },
        )
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.update(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(AutoGeneratedDraftTestCase, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.update(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(
            AsyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.list(
            evaluation_dataset_id="evaluation_dataset_id",
            check_missing_chunks=True,
            limit=1,
            page=1,
        )
        assert_matches_type(
            AsyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(
            AsyncPageResponse[AutogeneratedDraftTestCaseListResponse], autogenerated_draft_test_case, path=["response"]
        )

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(
                AsyncPageResponse[AutogeneratedDraftTestCaseListResponse],
                autogenerated_draft_test_case,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.list(
                evaluation_dataset_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.delete(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(GenericDeleteResponse, autogenerated_draft_test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.delete(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_approve(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_method_approve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        autogenerated_draft_test_case = await async_client.evaluation_datasets.autogenerated_draft_test_cases.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
            force=True,
        )
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_raw_response_approve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        autogenerated_draft_test_case = await response.parse()
        assert_matches_type(ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"])

    @parametrize
    async def test_streaming_response_approve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.autogenerated_draft_test_cases.with_streaming_response.approve(
            autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            autogenerated_draft_test_case = await response.parse()
            assert_matches_type(
                ApproveAutoGeneratedDraftTestCaseResponse, autogenerated_draft_test_case, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_approve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
                autogenerated_draft_test_case_id="autogenerated_draft_test_case_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `autogenerated_draft_test_case_id` but received ''"
        ):
            await async_client.evaluation_datasets.autogenerated_draft_test_cases.with_raw_response.approve(
                autogenerated_draft_test_case_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .message_update_params import MessageUpdateParams as MessageUpdateParams
from .message_update_response import MessageUpdateResponse as MessageUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["CompletionResponse", "Choice", "ChoiceLogprobs", "ChoiceLogprobsContent", "ChoiceLogprobsContentTopLogprob"]


class ChoiceLogprobsContentTopLogprob(BaseModel):
    token: str

    logprob: float

    bytes: Optional[List[int]] = None


class ChoiceLogprobsContent(BaseModel):
    token: str

    logprob: float

    top_logprobs: List[ChoiceLogprobsContentTopLogprob]

    bytes: Optional[List[int]] = None


class ChoiceLogprobs(BaseModel):
    content: Optional[List[ChoiceLogprobsContent]] = None


class Choice(BaseModel):
    index: int

    logprobs: Optional[ChoiceLogprobs] = None


class CompletionResponse(BaseModel):
    completions: List[List[object]]

    choices: Optional[List[Choice]] = None

    finish_reason: Optional[str] = None

    prompt_tokens: Optional[int] = None

    response_tokens: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CompletionsResponse", "Completion", "TokenUsage"]


class Completion(BaseModel):
    text: str
    """Completion text. If streaming, this field will contain each packet of text."""

    finish_reason: Optional[str] = None
    """Reason the LLM finished generating text."""

    response_metadata: Optional[object] = None
    """Additional metadata returned from the completion response"""


class TokenUsage(BaseModel):
    total: int
    """Total number of tokens in both the prompt and the completion."""

    completion: Optional[int] = None
    """Number of tokens in the completion."""

    prompt: Optional[int] = None
    """Number of tokens in the prompt."""


class CompletionsResponse(BaseModel):
    completion: Completion
    """The actual completion text and the finish reason."""

    token_usage: Optional[TokenUsage] = None
    """Token usage numbers.

    If streaming, this field is null until the stream completes, at which point it
    will be populated (if supported).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, overload
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge_bases import upload_list_params, upload_create_params, upload_retrieve_params
from ...types.local_chunks_source_config_param import LocalChunksSourceConfigParam
from ...types.knowledge_bases.upload_list_response import UploadListResponse
from ...types.knowledge_bases.knowledge_base_upload import KnowledgeBaseUpload
from ...types.knowledge_bases.cancel_knowledge_base_upload_response import CancelKnowledgeBaseUploadResponse
from ...types.knowledge_bases.create_knowledge_base_upload_response import CreateKnowledgeBaseUploadResponse

__all__ = ["UploadsResource", "AsyncUploadsResource"]


class UploadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return UploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return UploadsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig
        | NotGiven = NOT_GIVEN,
        data_source_auth_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          data_source_config: Configuration for the data source which describes where to find the data.

          chunking_strategy_config: Configuration for the chunking strategy which describes how to chunk the data.

          data_source_auth_config: Configuration for the data source which describes how to authenticate to the
              data source.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: LocalChunksSourceConfigParam,
        chunks: Iterable[upload_create_params.CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk]
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          data_source_config: Configuration for the data source which describes where to find the data.

          chunks: List of chunks.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceIDRequestChunkingStrategyConfig,
        data_source_id: str,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          chunking_strategy_config: Configuration for the chunking strategy which describes how to chunk the data.

          data_source_id: Id of the data source to fetch.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["data_source_config"], ["chunking_strategy_config", "data_source_id"])
    def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig
        | LocalChunksSourceConfigParam
        | NotGiven = NOT_GIVEN,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig
        | NotGiven = NOT_GIVEN,
        data_source_auth_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        chunks: Iterable[upload_create_params.CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk]
        | NotGiven = NOT_GIVEN,
        data_source_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads",
            body=maybe_transform(
                {
                    "data_source_config": data_source_config,
                    "chunking_strategy_config": chunking_strategy_config,
                    "data_source_auth_config": data_source_auth_config,
                    "force_reupload": force_reupload,
                    "chunks": chunks,
                    "data_source_id": data_source_id,
                },
                upload_create_params.UploadCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseUploadResponse,
        )

    def retrieve(
        self,
        upload_id: str,
        *,
        knowledge_base_id: str,
        include_artifact_list: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseUpload:
        """
        Get Upload Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_id:
            raise ValueError(f"Expected a non-empty value for `upload_id` but received {upload_id!r}")
        return self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads/{upload_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_artifact_list": include_artifact_list}, upload_retrieve_params.UploadRetrieveParams
                ),
            ),
            cast_to=KnowledgeBaseUpload,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Running", "Completed", "Failed", "Canceled"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[UploadListResponse]:
        """
        ### Description

        List all uploads for a knowledge base.

        ### Details

        This API can be used to list all uploads that have been created for a knowledge
        base. This API will return the details of all uploads including their IDs, their
        statuses, the data source configs they use, the chunking strategy configs they
        use, and the timestamps for their creation and last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads",
            page=SyncPageResponse[UploadListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    upload_list_params.UploadListParams,
                ),
            ),
            model=UploadListResponse,
        )

    def cancel(
        self,
        upload_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CancelKnowledgeBaseUploadResponse:
        """
        Cancel Upload Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_id:
            raise ValueError(f"Expected a non-empty value for `upload_id` but received {upload_id!r}")
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads/{upload_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelKnowledgeBaseUploadResponse,
        )


class AsyncUploadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncUploadsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig
        | NotGiven = NOT_GIVEN,
        data_source_auth_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          data_source_config: Configuration for the data source which describes where to find the data.

          chunking_strategy_config: Configuration for the chunking strategy which describes how to chunk the data.

          data_source_auth_config: Configuration for the data source which describes how to authenticate to the
              data source.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: LocalChunksSourceConfigParam,
        chunks: Iterable[upload_create_params.CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk]
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          data_source_config: Configuration for the data source which describes where to find the data.

          chunks: List of chunks.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        knowledge_base_id: str,
        *,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceIDRequestChunkingStrategyConfig,
        data_source_id: str,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        """
        Submit Upload Job

        Args:
          chunking_strategy_config: Configuration for the chunking strategy which describes how to chunk the data.

          data_source_id: Id of the data source to fetch.

          force_reupload: Force reingest, regardless the change of the source file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["data_source_config"], ["chunking_strategy_config", "data_source_id"])
    async def create(
        self,
        knowledge_base_id: str,
        *,
        data_source_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceConfig
        | LocalChunksSourceConfigParam
        | NotGiven = NOT_GIVEN,
        chunking_strategy_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestChunkingStrategyConfig
        | NotGiven = NOT_GIVEN,
        data_source_auth_config: upload_create_params.CreateKnowledgeBaseV2UploadFromDataSourceRequestDataSourceAuthConfig
        | NotGiven = NOT_GIVEN,
        force_reupload: bool | NotGiven = NOT_GIVEN,
        chunks: Iterable[upload_create_params.CreateKnowledgeBaseV2UploadFromLocalChunksRequestChunk]
        | NotGiven = NOT_GIVEN,
        data_source_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateKnowledgeBaseUploadResponse:
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads",
            body=await async_maybe_transform(
                {
                    "data_source_config": data_source_config,
                    "chunking_strategy_config": chunking_strategy_config,
                    "data_source_auth_config": data_source_auth_config,
                    "force_reupload": force_reupload,
                    "chunks": chunks,
                    "data_source_id": data_source_id,
                },
                upload_create_params.UploadCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateKnowledgeBaseUploadResponse,
        )

    async def retrieve(
        self,
        upload_id: str,
        *,
        knowledge_base_id: str,
        include_artifact_list: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> KnowledgeBaseUpload:
        """
        Get Upload Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_id:
            raise ValueError(f"Expected a non-empty value for `upload_id` but received {upload_id!r}")
        return await self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads/{upload_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_artifact_list": include_artifact_list}, upload_retrieve_params.UploadRetrieveParams
                ),
            ),
            cast_to=KnowledgeBaseUpload,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Running", "Completed", "Failed", "Canceled"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[UploadListResponse, AsyncPageResponse[UploadListResponse]]:
        """
        ### Description

        List all uploads for a knowledge base.

        ### Details

        This API can be used to list all uploads that have been created for a knowledge
        base. This API will return the details of all uploads including their IDs, their
        statuses, the data source configs they use, the chunking strategy configs they
        use, and the timestamps for their creation and last-updated time.

        #### Backwards Compatibility

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads",
            page=AsyncPageResponse[UploadListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    upload_list_params.UploadListParams,
                ),
            ),
            model=UploadListResponse,
        )

    async def cancel(
        self,
        upload_id: str,
        *,
        knowledge_base_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CancelKnowledgeBaseUploadResponse:
        """
        Cancel Upload Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not upload_id:
            raise ValueError(f"Expected a non-empty value for `upload_id` but received {upload_id!r}")
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/uploads/{upload_id}/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelKnowledgeBaseUploadResponse,
        )


class UploadsResourceWithRawResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create = to_raw_response_wrapper(
            uploads.create,
        )
        self.retrieve = to_raw_response_wrapper(
            uploads.retrieve,
        )
        self.list = to_raw_response_wrapper(
            uploads.list,
        )
        self.cancel = to_raw_response_wrapper(
            uploads.cancel,
        )


class AsyncUploadsResourceWithRawResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create = async_to_raw_response_wrapper(
            uploads.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            uploads.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            uploads.list,
        )
        self.cancel = async_to_raw_response_wrapper(
            uploads.cancel,
        )


class UploadsResourceWithStreamingResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create = to_streamed_response_wrapper(
            uploads.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            uploads.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            uploads.list,
        )
        self.cancel = to_streamed_response_wrapper(
            uploads.cancel,
        )


class AsyncUploadsResourceWithStreamingResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create = async_to_streamed_response_wrapper(
            uploads.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            uploads.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            uploads.list,
        )
        self.cancel = async_to_streamed_response_wrapper(
            uploads.cancel,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AzureBlobStorageDataSourceConfigParam"]


class AzureBlobStorageDataSourceConfigParam(TypedDict, total=False):
    container_url: Required[str]
    """
    The full URL of the container such as
    'https://your-account-name.blob.core.windows.net/your-container-name'
    """

    source: Required[Literal["AzureBlobStorage"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["LocalChunksSourceConfigParam"]


class LocalChunksSourceConfigParam(TypedDict, total=False):
    artifact_name: Required[str]
    """The file name assigned to the artifact, containing a file extension.

    Adding an extension is mandatory, to allow detecting file types for text
    extraction.
    """

    artifact_uri: Required[str]
    """
    A unique identifier for an artifact within the knowledge base, such as full path
    in a directory or file system.
    """

    source: Required[Literal["LocalChunks"]]

    deduplication_strategy: Literal["Overwrite", "Fail"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "FineTuningJob",
    "VendorConfiguration",
    "VendorConfigurationLaunchFineTuningJobConfiguration",
    "VendorConfigurationLlmEngineFineTuningJobConfiguration",
    "VendorConfigurationOpenAIFineTuningJobConfiguration",
]


class VendorConfigurationLaunchFineTuningJobConfiguration(BaseModel):
    hyperparameters: Optional[object] = None

    output: Optional[str] = None

    suffix: Optional[str] = None

    vendor: Optional[Literal["LAUNCH"]] = None

    wandb_config: Optional[object] = None


class VendorConfigurationLlmEngineFineTuningJobConfiguration(BaseModel):
    hyperparameters: Optional[object] = None

    output: Optional[str] = None

    suffix: Optional[str] = None

    vendor: Optional[Literal["LLMENGINE"]] = None

    wandb_config: Optional[object] = None


class VendorConfigurationOpenAIFineTuningJobConfiguration(BaseModel):
    hyperparameters: Optional[object] = None

    suffix: Optional[str] = None

    vendor: Optional[Literal["OPENAI"]] = None


VendorConfiguration: TypeAlias = Annotated[
    Union[
        VendorConfigurationLaunchFineTuningJobConfiguration,
        VendorConfigurationLlmEngineFineTuningJobConfiguration,
        VendorConfigurationOpenAIFineTuningJobConfiguration,
    ],
    PropertyInfo(discriminator="vendor"),
]


class FineTuningJob(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    status: Literal["PENDING", "COMPLETED", "FAILED", "RUNNING", "CANCELED"]
    """An enumeration."""

    training_dataset_id: str

    base_model_id: Optional[str] = None

    fine_tuned_model_id: Optional[str] = None

    validation_dataset_id: Optional[str] = None

    vendor_configuration: Optional[VendorConfiguration] = None
